# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - EventAggregatorSummary Action

    @copyright: 2008, 2009 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2003-2008 MoinMoin:ThomasWaldmann,
                2004-2006 MoinMoin:AlexanderSchremmer,
                2007 MoinMoin:ReimarBauer.
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin import config
from MoinMoin.Page import Page
import EventAggregatorSupport

Dependencies = ['pages']

# Action class and supporting functions.

class EventAggregatorSummary(ActionBase):

    "A summary dialogue requesting various parameters."

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request

        category_list = []

        for category_name, category_pagename in \
            EventAggregatorSupport.getCategoryMapping(
                EventAggregatorSupport.getCategories(request),
                request):

            category_list.append('<option value="%s">%s</option>' % (category_pagename, category_name))

        month_list = []
        month_list.append('<option value=""></option>')

        for month in range(1, 13):
            month_label = _(EventAggregatorSupport.getMonthLabel(month))
            month_list.append('<option value="%02d">%s</option>' % (month, month_label))

        year_label = []

        d = {
            "buttons_html" : buttons_html,
            "category_label" : _("Categories"),
            "category_list" : "\n".join(category_list),
            "month_list" : "\n".join(month_list),
            "start_label" : _("Start year and month"),
            "start_year_default" : "",
            "end_label" : _("End year and month"),
            "end_year_default" : "",
            }

        return '''
<table>
    <tr>
        <td class="label"><label>%(category_label)s</label></td>
        <td class="content">
            <select multiple="multiple" name="category">
                %(category_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(start_label)s</label></td>
        <td>
            <select name="start-month">
                %(month_list)s
            </select>
            <input name="start-year" type="text" value="%(start_year_default)s" size="4" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(end_label)s</label></td>
        <td>
            <select name="end-month">
                %(month_list)s
            </select>
            <input name="end-year" type="text" value="%(end_year_default)s" size="4" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>
''' % d

    def do_action(self):

        "Write the iCalendar resource."

        _ = self._
        form = self.request.form

        # If no category names exist in the request, an error message is
        # returned.

        category_names = form.get("category", [])

        if not category_names:
            return 0, _("No categories specified.")

        write_resource(self.request)
        return 1, None

    def render_success(self, msg, msgtype):

        """
        Render neither 'msg' nor 'msgtype' since a resource has already been
        produced.
        """

        pass

def getQuotedText(text):

    "Return the 'text' quoted for iCalendar purposes."

    return text.replace(";", r"\;").replace(",", r"\,")

def write_resource(request):

    """
    For the given 'request', write an iCalendar summary of the event data found
    in the categories specified via the "category" request parameter, using the
    "start" and "end" parameters (if specified). Multiple "category" parameters
    can be specified.
    """

    category_names = request.form.get("category", [])

    # Otherwise, produce an iCalendar resource.

    calendar_start = EventAggregatorSupport.getFormMonth(request, None, "start")
    calendar_end = EventAggregatorSupport.getFormMonth(request, None, "end")

    # Look for separate start and end years and months.

    form = request.form

    if calendar_start is None:
        calendar_start = EventAggregatorSupport.getFormMonthPair(request, "start-year", "start-month")

    if calendar_end is None:
        calendar_end = EventAggregatorSupport.getFormMonthPair(request, "end-year", "end-month")

    events, shown_events, all_shown_events, earliest, latest = \
        EventAggregatorSupport.getEvents(request, category_names, calendar_start, calendar_end)

    # Output iCalendar data...

    request.emit_http_headers(["Content-Type: text/calendar; charset=%s" % config.charset])

    request.write("BEGIN:VCALENDAR\r\n")
    request.write("PRODID:-//MoinMoin//EventAggregatorSummary\r\n")
    request.write("VERSION:2.0\r\n")

    for event_page, event_details in all_shown_events:

        # Get the summary and timestamp details.

        event_summary = EventAggregatorSupport.getEventSummary(event_page, event_details)

        # Get the initial revision of the page.

        revisions = event_page.getRevList()
        event_page_initial = Page(request, event_page.page_name, rev=revisions[-1])

        # Get the created and last modified times.

        created = EventAggregatorSupport.getPageDate(event_page_initial)
        last_modified = EventAggregatorSupport.getPageDate(event_page)
        sequence = len(revisions) - 1

        # Output the event details.

        request.write("BEGIN:VEVENT\r\n")
        request.write("UID:%s\r\n" % request.getQualifiedURL(event_page.url(request)))
        request.write("URL:%s\r\n" % request.getQualifiedURL(event_page.url(request)))
        request.write("DTSTAMP:%04d%02d%02dT%02d%02d%02dZ\r\n" % created[:6])
        request.write("LAST-MODIFIED:%04d%02d%02dT%02d%02d%02dZ\r\n" % last_modified[:6])
        request.write("SEQUENCE:%d\r\n" % sequence)
        request.write("DTSTART;VALUE=DATE:%04d%02d%02d\r\n" % event_details["start"])
        request.write("DTEND;VALUE=DATE:%04d%02d%02d\r\n" % EventAggregatorSupport.nextdate(event_details["end"]))
        request.write("SUMMARY:%s\r\n" % getQuotedText(event_summary))

        # Optional details.

        if event_details.has_key("topics") or event_details.has_key("categories"):
            request.write("CATEGORIES:%s\r\n" % ",".join(
                [getQuotedText(topic) for topic in event_details.get("topics") or event_details.get("categories")]
                ))
        if event_details.has_key("location"):
            request.write("LOCATION:%s\r\n" % getQuotedText(event_details["location"]))

        request.write("END:VEVENT\r\n")

    request.write("END:VCALENDAR\r\n")

# Action function.

def execute(pagename, request):
    EventAggregatorSummary(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
